@@ -1,15 +1,19 @@
-/***********************************************************************/
-/*                                                                     */
-/*                                OCaml                                */
-/*                                                                     */
-/*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         */
-/*                                                                     */
-/*  Copyright 1996 Institut National de Recherche en Informatique et   */
-/*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License, with    */
-/*  the special exception on linking described in file ../LICENSE.     */
-/*                                                                     */
-/***********************************************************************/
+/**************************************************************************/
+/*                                                                        */
+/*                                 OCaml                                  */
+/*                                                                        */
+/*             Xavier Leroy, projet Cristal, INRIA Rocquencourt           */
+/*                                                                        */
+/*   Copyright 1996 Institut National de Recherche en Informatique et     */
+/*     en Automatique.                                                    */
+/*                                                                        */
+/*   All rights reserved.  This file is distributed under the terms of    */
+/*   the GNU Lesser General Public License version 2.1, with the          */
+/*   special exception on linking described in the file LICENSE.          */
+/*                                                                        */
+/**************************************************************************/
+
+#define CAML_INTERNALS
 
 /* The generic hashing primitive */
 
@@ -41,7 +45,7 @@
   h *= 0xc2b2ae35; \
   h ^= h >> 16;
 
-CAMLexport uint32 caml_hash_mix_uint32(uint32 h, uint32 d)
+CAMLexport uint32_t caml_hash_mix_uint32(uint32_t h, uint32_t d)
 {
   MIX(h, d);
   return h;
@@ -49,17 +53,17 @@ CAMLexport uint32 caml_hash_mix_uint32(uint32 h, uint32 d)
 
 /* Mix a platform-native integer. */
 
-CAMLexport uint32 caml_hash_mix_intnat(uint32 h, intnat d)
+CAMLexport uint32_t caml_hash_mix_intnat(uint32_t h, intnat d)
 {
-  uint32 n;
+  uint32_t n;
 #ifdef ARCH_SIXTYFOUR
   /* Mix the low 32 bits and the high 32 bits, in a way that preserves
-     32/64 compatibility: we want n = (uint32) d
+     32/64 compatibility: we want n = (uint32_t) d
      if d is in the range [-2^31, 2^31-1]. */
   n = (d >> 32) ^ (d >> 63) ^ d;
   /* If 0 <= d < 2^31:   d >> 32 = 0     d >> 63 = 0
      If -2^31 <= d < 0:  d >> 32 = -1    d >> 63 = -1
-     In both cases, n = (uint32) d.  */
+     In both cases, n = (uint32_t) d.  */
 #else
   n = d;
 #endif
@@ -69,9 +73,9 @@ CAMLexport uint32 caml_hash_mix_intnat(uint32 h, intnat d)
 
 /* Mix a 64-bit integer. */
 
-CAMLexport uint32 caml_hash_mix_int64(uint32 h, int64 d)
+CAMLexport uint32_t caml_hash_mix_int64(uint32_t h, int64_t d)
 {
-  uint32 hi = (uint32) (d >> 32), lo = (uint32) d;
+  uint32_t hi = (uint32_t) (d >> 32), lo = (uint32_t) d;
   MIX(h, lo);
   MIX(h, hi);
   return h;
@@ -82,17 +86,17 @@ CAMLexport uint32 caml_hash_mix_int64(uint32 h, int64 d)
    Treats all NaNs identically.
 */
 
-CAMLexport uint32 caml_hash_mix_double(uint32 hash, double d)
+CAMLexport uint32_t caml_hash_mix_double(uint32_t hash, double d)
 {
   union {
     double d;
 #if defined(ARCH_BIG_ENDIAN) || (defined(__arm__) && !defined(__ARM_EABI__))
-    struct { uint32 h; uint32 l; } i;
+    struct { uint32_t h; uint32_t l; } i;
 #else
-    struct { uint32 l; uint32 h; } i;
+    struct { uint32_t l; uint32_t h; } i;
 #endif
   } u;
-  uint32 h, l;
+  uint32_t h, l;
   /* Convert to two 32-bit halves */
   u.d = d;
   h = u.i.h; l = u.i.l;
@@ -115,14 +119,14 @@ CAMLexport uint32 caml_hash_mix_double(uint32 hash, double d)
    Treats all NaNs identically.
 */
 
-CAMLexport uint32 caml_hash_mix_float(uint32 hash, float d)
+CAMLexport uint32_t caml_hash_mix_float(uint32_t hash, float d)
 {
   union {
     float f;
-    uint32 i;
+    uint32_t i;
   } u;
-  uint32 n;
-  /* Convert to int32 */
+  uint32_t n;
+  /* Convert to int32_t */
   u.f = d;  n = u.i;
   /* Normalize NaNs */
   if ((n & 0x7F800000) == 0x7F800000 && (n & 0x007FFFFF) != 0) {
@@ -138,11 +142,11 @@ CAMLexport uint32 caml_hash_mix_float(uint32 hash, float d)
 
 /* Mix an OCaml string */
 
-CAMLexport uint32 caml_hash_mix_string(uint32 h, value s)
+CAMLexport uint32_t caml_hash_mix_string(uint32_t h, value s)
 {
   mlsize_t len = caml_string_length(s);
   mlsize_t i;
-  uint32 w;
+  uint32_t w;
 
   /* Mix by 32-bit blocks (little-endian) */
   for (i = 0; i + 4 <= len; i += 4) {
@@ -152,7 +156,7 @@ CAMLexport uint32 caml_hash_mix_string(uint32 h, value s)
         | (Byte_u(s, i+2) << 16)
         | (Byte_u(s, i+3) << 24);
 #else
-    w = *((uint32 *) &Byte_u(s, i));
+    w = *((uint32_t *) &Byte_u(s, i));
 #endif
     MIX(h, w);
   }
@@ -166,7 +170,7 @@ CAMLexport uint32 caml_hash_mix_string(uint32 h, value s)
   default: /*skip*/;     /* len & 3 == 0, no extra bytes, do nothing */
   }
   /* Finally, mix in the length.  Ignore the upper 32 bits, generally 0. */
-  h ^= (uint32) len;
+  h ^= (uint32_t) len;
   return h;
 }
 
@@ -184,7 +188,7 @@ CAMLprim value caml_hash(value count, value limit, value seed, value obj)
   intnat wr;                    /* One past position of last value in queue */
   intnat sz;                    /* Max number of values to put in queue */
   intnat num;                   /* Max number of meaningful values to see */
-  uint32 h;                     /* Rolling hash */
+  uint32_t h;                     /* Rolling hash */
   value v;
   mlsize_t i, len;
 
@@ -245,7 +249,7 @@ CAMLprim value caml_hash(value count, value limit, value seed, value obj)
         /* If no hashing function provided, do nothing. */
         /* Only use low 32 bits of custom hash, for 32/64 compatibility */
         if (Custom_ops_val(v)->hash != NULL) {
-          uint32 n = (uint32) Custom_ops_val(v)->hash(v);
+          uint32_t n = (uint32_t) Custom_ops_val(v)->hash(v);
           h = caml_hash_mix_uint32(h, n);
           num--;
         }
@@ -408,5 +412,5 @@ CAMLexport value caml_hash_variant(char const * tag)
 #endif
   /* Force sign extension of bit 31 for compatibility between 32 and 64-bit
      platforms */
-  return (int32) accu;
+  return (int32_t) accu;
 }
