@@ -1,15 +1,19 @@
-/***********************************************************************/
-/*                                                                     */
-/*                                OCaml                                */
-/*                                                                     */
-/*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         */
-/*                                                                     */
-/*  Copyright 1996 Institut National de Recherche en Informatique et   */
-/*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License, with    */
-/*  the special exception on linking described in file ../LICENSE.     */
-/*                                                                     */
-/***********************************************************************/
+/**************************************************************************/
+/*                                                                        */
+/*                                 OCaml                                  */
+/*                                                                        */
+/*             Xavier Leroy, projet Cristal, INRIA Rocquencourt           */
+/*                                                                        */
+/*   Copyright 1996 Institut National de Recherche en Informatique et     */
+/*     en Automatique.                                                    */
+/*                                                                        */
+/*   All rights reserved.  This file is distributed under the terms of    */
+/*   the GNU Lesser General Public License version 2.1, with the          */
+/*   special exception on linking described in the file LICENSE.          */
+/*                                                                        */
+/**************************************************************************/
+
+#define CAML_INTERNALS
 
 /* Structured input, compact format */
 
@@ -19,6 +23,7 @@
 #include <stdio.h>
 #include "caml/alloc.h"
 #include "caml/callback.h"
+#include "caml/config.h"
 #include "caml/custom.h"
 #include "caml/fail.h"
 #include "caml/gc.h"
@@ -33,24 +38,20 @@
 static unsigned char * intern_src;
 /* Reading pointer in block holding input data. */
 
-static unsigned char * intern_input;
-/* Pointer to beginning of block holding input data.
-   Meaningful only if intern_input_malloced = 1. */
-
-static int intern_input_malloced;
-/* 1 if intern_input was allocated by caml_stat_alloc()
-   and needs caml_stat_free() on error, 0 otherwise. */
+static unsigned char * intern_input = NULL;
+/* Pointer to beginning of block holding input data,
+   if non-NULL this pointer will be freed by the cleanup function. */
 
 static header_t * intern_dest;
 /* Writing pointer in destination block */
 
-static char * intern_extra_block;
+static char * intern_extra_block = NULL;
 /* If non-NULL, point to new heap chunk allocated with caml_alloc_for_heap. */
 
 static asize_t obj_counter;
 /* Count how many objects seen so far */
 
-static value * intern_obj_table;
+static value * intern_obj_table = NULL;
 /* The pointers to objects already seen */
 
 static unsigned int intern_color;
@@ -60,61 +61,109 @@ static header_t intern_header;
 /* Original header of the destination block.
    Meaningful only if intern_extra_block is NULL. */
 
-static value intern_block;
+static value intern_block = 0;
 /* Point to the heap block allocated as destination block.
    Meaningful only if intern_extra_block is NULL. */
 
 static char * intern_resolve_code_pointer(unsigned char digest[16],
                                           asize_t offset);
-static void intern_bad_code_pointer(unsigned char digest[16]) Noreturn;
+
+CAMLnoreturn_start
+static void intern_bad_code_pointer(unsigned char digest[16])
+CAMLnoreturn_end;
 
 static void intern_free_stack(void);
 
-#define Sign_extend_shift ((sizeof(intnat) - 1) * 8)
-#define Sign_extend(x) (((intnat)(x) << Sign_extend_shift) >> Sign_extend_shift)
+static inline unsigned char read8u(void)
+{ return *intern_src++; }
 
-#define read8u() (*intern_src++)
-#define read8s() Sign_extend(*intern_src++)
-#define read16u() \
-  (intern_src += 2, \
-   (intern_src[-2] << 8) + intern_src[-1])
-#define read16s() \
-  (intern_src += 2, \
-   (Sign_extend(intern_src[-2]) << 8) + intern_src[-1])
-#define read32u() \
-  (intern_src += 4, \
-   ((uintnat)(intern_src[-4]) << 24) + (intern_src[-3] << 16) + \
-   (intern_src[-2] << 8) + intern_src[-1])
-#define read32s() \
-  (intern_src += 4, \
-   (Sign_extend(intern_src[-4]) << 24) + (intern_src[-3] << 16) + \
-   (intern_src[-2] << 8) + intern_src[-1])
+static inline signed char read8s(void)
+{ return *intern_src++; }
+
+static inline uint16_t read16u(void)
+{
+  uint16_t res = (intern_src[0] << 8) + intern_src[1];
+  intern_src += 2;
+  return res;
+}
+
+static inline int16_t read16s(void)
+{
+  int16_t res = (intern_src[0] << 8) + intern_src[1];
+  intern_src += 2;
+  return res;
+}
+
+static inline uint32_t read32u(void)
+{
+  uint32_t res =
+    ((uint32_t)(intern_src[0]) << 24) + (intern_src[1] << 16)
+    + (intern_src[2] << 8) + intern_src[3];
+  intern_src += 4;
+  return res;
+}
+
+static inline int32_t read32s(void)
+{
+  int32_t res =
+    ((uint32_t)(intern_src[0]) << 24) + (intern_src[1] << 16)
+    + (intern_src[2] << 8) + intern_src[3];
+  intern_src += 4;
+  return res;
+}
 
 #ifdef ARCH_SIXTYFOUR
-static intnat read64s(void)
+static uintnat read64u(void)
 {
-  intnat res;
-  int i;
-  res = 0;
-  for (i = 0; i < 8; i++) res = (res << 8) + intern_src[i];
+  uintnat res =
+    ((uintnat) (intern_src[0]) << 56)
+    + ((uintnat) (intern_src[1]) << 48)
+    + ((uintnat) (intern_src[2]) << 40)
+    + ((uintnat) (intern_src[3]) << 32)
+    + ((uintnat) (intern_src[4]) << 24)
+    + ((uintnat) (intern_src[5]) << 16)
+    + ((uintnat) (intern_src[6]) << 8)
+    + (uintnat) (intern_src[7]);
   intern_src += 8;
   return res;
 }
 #endif
 
-#define readblock(dest,len) \
-  (memmove((dest), intern_src, (len)), intern_src += (len))
+static inline void readblock(void * dest, intnat len)
+{
+  memcpy(dest, intern_src, len);
+  intern_src += len;
+}
+
+static void intern_init(void * src, void * input)
+{
+  /* This is asserted at the beginning of demarshaling primitives.
+     If it fails, it probably means that an exception was raised
+     without calling intern_cleanup() during the previous demarshaling. */
+  Assert (intern_input == NULL && intern_obj_table == NULL \
+     && intern_extra_block == NULL && intern_block == 0);
+  intern_src = src;
+  intern_input = input;
+}
 
 static void intern_cleanup(void)
 {
-  if (intern_input_malloced) caml_stat_free(intern_input);
-  if (intern_obj_table != NULL) caml_stat_free(intern_obj_table);
+  if (intern_input != NULL) {
+     caml_stat_free(intern_input);
+     intern_input = NULL;
+  }
+  if (intern_obj_table != NULL) {
+    caml_stat_free(intern_obj_table);
+    intern_obj_table = NULL;
+  }
   if (intern_extra_block != NULL) {
     /* free newly allocated heap chunk */
     caml_free_for_heap(intern_extra_block);
+    intern_extra_block = NULL;
   } else if (intern_block != 0) {
     /* restore original header for heap block, otherwise GC is confused */
     Hd_val(intern_block) = intern_header;
+    intern_block = 0;
   }
   /* free the recursion stack */
   intern_free_stack();
@@ -143,6 +192,7 @@ static void readfloat(double * dest, unsigned int code)
 #endif
 }
 
+/* [len] is a number of floats */
 static void readfloats(double * dest, mlsize_t len, unsigned int code)
 {
   mlsize_t i;
@@ -287,7 +337,7 @@ static void intern_rec(value *dest)
   case OFreshOID:
     /* Refresh the object ID */
     /* but do not do it for predefined exception slots */
-    if (Int_val(Field((value)dest, 1)) >= 0)
+    if (Long_val(Field((value)dest, 1)) >= 0)
       caml_set_oo_id((value)dest);
     /* Pop item and iterate */
     sp--;
@@ -315,7 +365,7 @@ static void intern_rec(value *dest)
       } else {
         v = Val_hp(intern_dest);
         if (intern_obj_table != NULL) intern_obj_table[obj_counter++] = v;
-        *intern_dest = Make_header(size, tag, intern_color);
+        *intern_dest = Make_header_allocated_here(size, tag, intern_color);
         intern_dest += 1 + size;
         /* For objects, we need to freshen the oid */
         if (tag == Object_tag) {
@@ -345,7 +395,7 @@ static void intern_rec(value *dest)
       size = (len + sizeof(value)) / sizeof(value);
       v = Val_hp(intern_dest);
       if (intern_obj_table != NULL) intern_obj_table[obj_counter++] = v;
-      *intern_dest = Make_header(size, String_tag, intern_color);
+      *intern_dest = Make_header_allocated_here(size, String_tag, intern_color);
       intern_dest += 1 + size;
       Field(v, size - 1) = 0;
       ofs_ind = Bsize_wsize(size) - 1;
@@ -364,7 +414,7 @@ static void intern_rec(value *dest)
         break;
       case CODE_INT64:
 #ifdef ARCH_SIXTYFOUR
-        v = Val_long(read64s());
+        v = Val_long((intnat) (read64u()));
         break;
 #else
         intern_cleanup();
@@ -385,21 +435,22 @@ static void intern_rec(value *dest)
       case CODE_SHARED32:
         ofs = read32u();
         goto read_shared;
+#ifdef ARCH_SIXTYFOUR
+      case CODE_SHARED64:
+        ofs = read64u();
+        goto read_shared;
+#endif
       case CODE_BLOCK32:
         header = (header_t) read32u();
         tag = Tag_hd(header);
         size = Wosize_hd(header);
         goto read_block;
-      case CODE_BLOCK64:
 #ifdef ARCH_SIXTYFOUR
-        header = (header_t) read64s();
+      case CODE_BLOCK64:
+        header = (header_t) read64u();
         tag = Tag_hd(header);
         size = Wosize_hd(header);
         goto read_block;
-#else
-        intern_cleanup();
-        caml_failwith("input_value: data block too large");
-        break;
 #endif
       case CODE_STRING8:
         len = read8u();
@@ -407,11 +458,17 @@ static void intern_rec(value *dest)
       case CODE_STRING32:
         len = read32u();
         goto read_string;
+#ifdef ARCH_SIXTYFOUR
+      case CODE_STRING64:
+        len = read64u();
+        goto read_string;
+#endif
       case CODE_DOUBLE_LITTLE:
       case CODE_DOUBLE_BIG:
         v = Val_hp(intern_dest);
         if (intern_obj_table != NULL) intern_obj_table[obj_counter++] = v;
-        *intern_dest = Make_header(Double_wosize, Double_tag, intern_color);
+        *intern_dest = Make_header_allocated_here(Double_wosize, Double_tag,
+                                                  intern_color);
         intern_dest += 1 + Double_wosize;
         readfloat((double *) v, code);
         break;
@@ -422,7 +479,8 @@ static void intern_rec(value *dest)
         size = len * Double_wosize;
         v = Val_hp(intern_dest);
         if (intern_obj_table != NULL) intern_obj_table[obj_counter++] = v;
-        *intern_dest = Make_header(size, Double_array_tag, intern_color);
+        *intern_dest = Make_header_allocated_here(size, Double_array_tag,
+                                                  intern_color);
         intern_dest += 1 + size;
         readfloats((double *) v, len, code);
         break;
@@ -430,6 +488,12 @@ static void intern_rec(value *dest)
       case CODE_DOUBLE_ARRAY32_BIG:
         len = read32u();
         goto read_double_array;
+#ifdef ARCH_SIXTYFOUR
+      case CODE_DOUBLE_ARRAY64_LITTLE:
+      case CODE_DOUBLE_ARRAY64_BIG:
+        len = read64u();
+        goto read_double_array;
+#endif
       case CODE_CODEPOINTER:
         ofs = read32u();
         readblock(digest, 16);
@@ -467,8 +531,15 @@ static void intern_rec(value *dest)
         size = 1 + (size + sizeof(value) - 1) / sizeof(value);
         v = Val_hp(intern_dest);
         if (intern_obj_table != NULL) intern_obj_table[obj_counter++] = v;
-        *intern_dest = Make_header(size, Custom_tag, intern_color);
+        *intern_dest = Make_header_allocated_here(size, Custom_tag,
+                                                  intern_color);
         Custom_ops_val(v) = ops;
+
+        if (ops->finalize != NULL && Is_young(v)) {
+          /* Remember that the block has a finalizer. */
+          add_to_custom_table (&caml_custom_table, v, 0, 1);
+        }
+
         intern_dest += 1 + size;
         break;
       default:
@@ -488,25 +559,30 @@ static void intern_rec(value *dest)
   intern_free_stack();
 }
 
-static void intern_alloc(mlsize_t whsize, mlsize_t num_objects)
+static void intern_alloc(mlsize_t whsize, mlsize_t num_objects,
+      int outside_heap)
 {
   mlsize_t wosize;
 
   if (whsize == 0) {
-    intern_obj_table = NULL;
-    intern_extra_block = NULL;
-    intern_block = 0;
+    Assert (intern_extra_block == NULL && intern_block == 0
+         && intern_obj_table == NULL);
     return;
   }
   wosize = Wosize_whsize(whsize);
-  if (wosize > Max_wosize) {
+  if (wosize > Max_wosize || outside_heap) {
     /* Round desired size up to next page */
     asize_t request =
       ((Bsize_wsize(whsize) + Page_size - 1) >> Page_log) << Page_log;
     intern_extra_block = caml_alloc_for_heap(request);
-    if (intern_extra_block == NULL) caml_raise_out_of_memory();
-    intern_color = caml_allocation_color(intern_extra_block);
+    if (intern_extra_block == NULL) {
+      intern_cleanup();
+      caml_raise_out_of_memory();
+    }
+    intern_color =
+      outside_heap ? Caml_black : caml_allocation_color(intern_extra_block);
     intern_dest = (header_t *) intern_extra_block;
+    Assert (intern_block == 0);
   } else {
     /* this is a specialised version of caml_alloc from alloc.c */
     if (wosize == 0){
@@ -514,21 +590,29 @@ static void intern_alloc(mlsize_t whsize, mlsize_t num_objects)
     }else if (wosize <= Max_young_wosize){
       intern_block = caml_alloc_small (wosize, String_tag);
     }else{
-      intern_block = caml_alloc_shr (wosize, String_tag);
+      intern_block = caml_alloc_shr_no_raise (wosize, String_tag);
       /* do not do the urgent_gc check here because it might darken
          intern_block into gray and break the Assert 3 lines down */
+      if (intern_block == 0) {
+        intern_cleanup();
+        caml_raise_out_of_memory();
+      }
     }
     intern_header = Hd_val(intern_block);
     intern_color = Color_hd(intern_header);
     Assert (intern_color == Caml_white || intern_color == Caml_black);
     intern_dest = (header_t *) Hp_val(intern_block);
-    intern_extra_block = NULL;
+    Assert (intern_extra_block == NULL);
   }
   obj_counter = 0;
-  if (num_objects > 0)
-    intern_obj_table = (value *) caml_stat_alloc(num_objects * sizeof(value));
-  else
-    intern_obj_table = NULL;
+  if (num_objects > 0) {
+    intern_obj_table = (value *) malloc(num_objects * sizeof(value));
+    if (intern_obj_table == NULL) {
+      intern_cleanup();
+      caml_raise_out_of_memory();
+    }
+  } else
+    Assert(intern_obj_table == NULL);
 }
 
 static void intern_add_to_heap(mlsize_t whsize)
@@ -536,10 +620,10 @@ static void intern_add_to_heap(mlsize_t whsize)
   /* Add new heap chunk to heap if needed */
   if (intern_extra_block != NULL) {
     /* If heap chunk not filled totally, build free block at end */
-    asize_t request =
-      ((Bsize_wsize(whsize) + Page_size - 1) >> Page_log) << Page_log;
+    asize_t request = Chunk_size (intern_extra_block);
     header_t * end_extra_block =
       (header_t *) intern_extra_block + Wsize_bsize(request);
+    Assert(intern_block == 0);
     Assert(intern_dest <= end_extra_block);
     if (intern_dest < end_extra_block){
       caml_make_free_blocks ((value *) intern_dest,
@@ -548,52 +632,123 @@ static void intern_add_to_heap(mlsize_t whsize)
     caml_allocated_words +=
       Wsize_bsize ((char *) intern_dest - intern_extra_block);
     caml_add_to_heap(intern_extra_block);
+    intern_extra_block = NULL; // To prevent intern_cleanup freeing it
+  } else {
+    intern_block = 0; // To prevent intern_cleanup rewriting its header
   }
 }
 
-value caml_input_val(struct channel *chan)
+/* Parsing the header */
+
+struct marshal_header {
+  uint32_t magic;
+  int header_len;
+  uintnat data_len;
+  uintnat num_objects;
+  uintnat whsize;
+};
+
+static void caml_parse_header(char * fun_name,
+                              /*out*/ struct marshal_header * h)
 {
-  uint32 magic;
-  mlsize_t block_len, num_objects, whsize;
+  char errmsg[100];
+
+  h->magic = read32u();
+  switch(h->magic) {
+  case Intext_magic_number_small:
+    h->header_len = 20;
+    h->data_len = read32u();
+    h->num_objects = read32u();
+#ifdef ARCH_SIXTYFOUR
+    read32u();
+    h->whsize = read32u();
+#else
+    h->whsize = read32u();
+    read32u();
+#endif
+    break;
+  case Intext_magic_number_big:
+#ifdef ARCH_SIXTYFOUR
+    h->header_len = 32;
+    read32u();
+    h->data_len = read64u();
+    h->num_objects = read64u();
+    h->whsize = read64u();
+#else
+    errmsg[sizeof(errmsg) - 1] = 0;
+    snprintf(errmsg, sizeof(errmsg) - 1,
+             "%s: object too large to be read back on a 32-bit platform",
+             fun_name);
+    caml_failwith(errmsg);
+#endif
+    break;
+  default:
+    errmsg[sizeof(errmsg) - 1] = 0;
+    snprintf(errmsg, sizeof(errmsg) - 1,
+             "%s: bad object",
+             fun_name);
+    caml_failwith(errmsg);
+  }
+}
+
+/* Reading from a channel */
+
+static value caml_input_val_core(struct channel *chan, int outside_heap)
+{
+  intnat r;
+  char header[32];
+  struct marshal_header h;
   char * block;
   value res;
 
   if (! caml_channel_binary_mode(chan))
     caml_failwith("input_value: not a binary channel");
-  magic = caml_getword(chan);
-  if (magic != Intext_magic_number) caml_failwith("input_value: bad object");
-  block_len = caml_getword(chan);
-  num_objects = caml_getword(chan);
-#ifdef ARCH_SIXTYFOUR
-  caml_getword(chan); /* skip size_32 */
-  whsize = caml_getword(chan);
-#else
-  whsize = caml_getword(chan);
-  caml_getword(chan); /* skip size_64 */
-#endif
+  /* Read and parse the header */
+  r = caml_really_getblock(chan, header, 20);
+  if (r == 0)
+    caml_raise_end_of_file();
+  else if (r < 20)
+    caml_failwith("input_value: truncated object");
+  intern_src = (unsigned char *) header;
+  if (read32u() == Intext_magic_number_big) {
+    /* Finish reading the header */
+    if (caml_really_getblock(chan, header + 20, 32 - 20) < 32 - 20)
+      caml_failwith("input_value: truncated object");
+  }
+  intern_src = (unsigned char *) header;
+  caml_parse_header("input_value", &h);
   /* Read block from channel */
-  block = caml_stat_alloc(block_len);
+  block = caml_stat_alloc(h.data_len);
   /* During [caml_really_getblock], concurrent [caml_input_val] operations
      can take place (via signal handlers or context switching in systhreads),
      and [intern_input] may change.  So, wait until [caml_really_getblock]
      is over before using [intern_input] and the other global vars. */
-  if (caml_really_getblock(chan, block, block_len) == 0) {
+  if (caml_really_getblock(chan, block, h.data_len) < h.data_len) {
     caml_stat_free(block);
     caml_failwith("input_value: truncated object");
   }
-  intern_input = (unsigned char *) block;
-  intern_input_malloced = 1;
-  intern_src = intern_input;
-  intern_alloc(whsize, num_objects);
+  /* Initialize global state */
+  intern_init(block, block);
+  intern_alloc(h.whsize, h.num_objects, outside_heap);
   /* Fill it in */
   intern_rec(&res);
-  intern_add_to_heap(whsize);
+  if (!outside_heap) {
+    intern_add_to_heap(h.whsize);
+  } else {
+    caml_disown_for_heap(intern_extra_block);
+    intern_extra_block = NULL;
+    intern_block = 0;
+  }
   /* Free everything */
-  caml_stat_free(intern_input);
-  if (intern_obj_table != NULL) caml_stat_free(intern_obj_table);
+  intern_cleanup();
   return caml_check_urgent_gc(res);
 }
 
+value caml_input_val(struct channel* chan)
+{
+  return caml_input_val_core(chan, 0);
+}
+
 CAMLprim value caml_input_value(value vchan)
 {
   CAMLparam1 (vchan);
@@ -606,30 +761,39 @@ CAMLprim value caml_input_value(value vchan)
   CAMLreturn (res);
 }
 
+/* Reading from memory-resident blocks */
+
+CAMLprim value caml_input_value_to_outside_heap(value vchan)
+{
+  CAMLparam1 (vchan);
+  struct channel * chan = Channel(vchan);
+  CAMLlocal1 (res);
+
+  Lock(chan);
+  res = caml_input_val_core(chan, 1);
+  Unlock(chan);
+  CAMLreturn (res);
+}
+
 CAMLexport value caml_input_val_from_string(value str, intnat ofs)
 {
   CAMLparam1 (str);
-  mlsize_t num_objects, whsize;
   CAMLlocal1 (obj);
+  struct marshal_header h;
 
-  intern_src = &Byte_u(str, ofs + 2*4);
-  intern_input_malloced = 0;
-  num_objects = read32u();
-#ifdef ARCH_SIXTYFOUR
-  intern_src += 4;  /* skip size_32 */
-  whsize = read32u();
-#else
-  whsize = read32u();
-  intern_src += 4;  /* skip size_64 */
-#endif
+  /* Initialize global state */
+  intern_init(&Byte_u(str, ofs), NULL);
+  caml_parse_header("input_val_from_string", &h);
+  if (ofs + h.header_len + h.data_len > caml_string_length(str))
+    caml_failwith("input_val_from_string: bad length");
   /* Allocate result */
-  intern_alloc(whsize, num_objects);
-  intern_src = &Byte_u(str, ofs + 5*4); /* If a GC occurred */
+  intern_alloc(h.whsize, h.num_objects, 0);
+  intern_src = &Byte_u(str, ofs + h.header_len); /* If a GC occurred */
   /* Fill it in */
   intern_rec(&obj);
-  intern_add_to_heap(whsize);
+  intern_add_to_heap(h.whsize);
   /* Free everything */
-  if (intern_obj_table != NULL) caml_stat_free(intern_obj_table);
+  intern_cleanup();
   CAMLreturn (caml_check_urgent_gc(obj));
 }
 
@@ -638,79 +802,77 @@ CAMLprim value caml_input_value_from_string(value str, value ofs)
   return caml_input_val_from_string(str, Long_val(ofs));
 }
 
-static value input_val_from_block(void)
+static value input_val_from_block(struct marshal_header * h)
 {
-  mlsize_t num_objects, whsize;
   value obj;
-
-  num_objects = read32u();
-#ifdef ARCH_SIXTYFOUR
-  intern_src += 4;  /* skip size_32 */
-  whsize = read32u();
-#else
-  whsize = read32u();
-  intern_src += 4;  /* skip size_64 */
-#endif
   /* Allocate result */
-  intern_alloc(whsize, num_objects);
+  intern_alloc(h->whsize, h->num_objects, 0);
   /* Fill it in */
   intern_rec(&obj);
-  intern_add_to_heap(whsize);
+  intern_add_to_heap(h->whsize);
   /* Free internal data structures */
-  if (intern_obj_table != NULL) caml_stat_free(intern_obj_table);
+  intern_cleanup();
   return caml_check_urgent_gc(obj);
 }
 
 CAMLexport value caml_input_value_from_malloc(char * data, intnat ofs)
 {
-  uint32 magic;
-  value obj;
+  struct marshal_header h;
 
-  intern_input = (unsigned char *) data;
-  intern_src = intern_input + ofs;
-  intern_input_malloced = 1;
-  magic = read32u();
-  if (magic != Intext_magic_number)
-    caml_failwith("input_value_from_malloc: bad object");
-  intern_src += 4;  /* Skip block_len */
-  obj = input_val_from_block();
-  /* Free the input */
-  caml_stat_free(intern_input);
-  return obj;
+  intern_init(data + ofs, data);
+
+  caml_parse_header("input_value_from_malloc", &h);
+
+  return input_val_from_block(&h);
 }
 
+/* [len] is a number of bytes */
 CAMLexport value caml_input_value_from_block(char * data, intnat len)
 {
-  uint32 magic;
-  mlsize_t block_len;
-  value obj;
+  struct marshal_header h;
 
-  intern_input = (unsigned char *) data;
-  intern_src = intern_input;
-  intern_input_malloced = 0;
-  magic = read32u();
-  if (magic != Intext_magic_number)
-    caml_failwith("input_value_from_block: bad object");
-  block_len = read32u();
-  if (5*4 + block_len > len)
-    caml_failwith("input_value_from_block: bad block length");
-  obj = input_val_from_block();
-  return obj;
+  /* Initialize global state */
+  intern_init(data, NULL);
+  caml_parse_header("input_value_from_block", &h);
+  if (h.header_len + h.data_len > len)
+    caml_failwith("input_val_from_block: bad length");
+  return input_val_from_block(&h);
 }
 
+/* [ofs] is a [value] that represents a number of bytes
+   result is a [value] that represents a number of bytes
+   To handle both the small and the big format,
+   we assume 20 bytes are available at [buff + ofs],
+   and we return the data size + the length of the part of the header
+   that remains to be read. */
+
 CAMLprim value caml_marshal_data_size(value buff, value ofs)
 {
-  uint32 magic;
-  mlsize_t block_len;
+  uint32_t magic;
+  int header_len;
+  uintnat data_len;
 
   intern_src = &Byte_u(buff, Long_val(ofs));
-  intern_input_malloced = 0;
   magic = read32u();
-  if (magic != Intext_magic_number){
+  switch(magic) {
+  case Intext_magic_number_small:
+    header_len = 20;
+    data_len = read32u();
+    break;
+  case Intext_magic_number_big:
+#ifdef ARCH_SIXTYFOUR
+    header_len = 32;
+    read32u();
+    data_len = read64u();
+#else
+    caml_failwith("Marshal.data_size: "
+                  "object too large to be read back on a 32-bit platform");
+#endif
+    break;
+  default:
     caml_failwith("Marshal.data_size: bad object");
   }
-  block_len = read32u();
-  return Val_long(block_len);
+  return Val_long((header_len - 20) + data_len);
 }
 
 /* Resolution of code pointers */
@@ -771,26 +933,26 @@ CAMLexport int caml_deserialize_sint_2(void)
   return read16s();
 }
 
-CAMLexport uint32 caml_deserialize_uint_4(void)
+CAMLexport uint32_t caml_deserialize_uint_4(void)
 {
   return read32u();
 }
 
-CAMLexport int32 caml_deserialize_sint_4(void)
+CAMLexport int32_t caml_deserialize_sint_4(void)
 {
   return read32s();
 }
 
-CAMLexport uint64 caml_deserialize_uint_8(void)
+CAMLexport uint64_t caml_deserialize_uint_8(void)
 {
-  uint64 i;
+  uint64_t i;
   caml_deserialize_block_8(&i, 1);
   return i;
 }
 
-CAMLexport int64 caml_deserialize_sint_8(void)
+CAMLexport int64_t caml_deserialize_sint_8(void)
 {
-  int64 i;
+  int64_t i;
   caml_deserialize_block_8(&i, 1);
   return i;
 }
@@ -811,7 +973,7 @@ CAMLexport double caml_deserialize_float_8(void)
 
 CAMLexport void caml_deserialize_block_1(void * data, intnat len)
 {
-  memmove(data, intern_src, len);
+  memcpy(data, intern_src, len);
   intern_src += len;
 }
 
@@ -823,7 +985,7 @@ CAMLexport void caml_deserialize_block_2(void * data, intnat len)
     Reverse_16(q, p);
   intern_src = p;
 #else
-  memmove(data, intern_src, len * 2);
+  memcpy(data, intern_src, len * 2);
   intern_src += len * 2;
 #endif
 }
@@ -836,7 +998,7 @@ CAMLexport void caml_deserialize_block_4(void * data, intnat len)
     Reverse_32(q, p);
   intern_src = p;
 #else
-  memmove(data, intern_src, len * 4);
+  memcpy(data, intern_src, len * 4);
   intern_src += len * 4;
 #endif
 }
@@ -849,7 +1011,7 @@ CAMLexport void caml_deserialize_block_8(void * data, intnat len)
     Reverse_64(q, p);
   intern_src = p;
 #else
-  memmove(data, intern_src, len * 8);
+  memcpy(data, intern_src, len * 8);
   intern_src += len * 8;
 #endif
 }
@@ -857,7 +1019,7 @@ CAMLexport void caml_deserialize_block_8(void * data, intnat len)
 CAMLexport void caml_deserialize_block_float_8(void * data, intnat len)
 {
 #if ARCH_FLOAT_ENDIANNESS == 0x01234567
-  memmove(data, intern_src, len * 8);
+  memcpy(data, intern_src, len * 8);
   intern_src += len * 8;
 #elif ARCH_FLOAT_ENDIANNESS == 0x76543210
   unsigned char * p, * q;
