@@ -1,18 +1,20 @@
-(***********************************************************************)
-(*                                                                     *)
-(*                             OCamldoc                                *)
-(*                                                                     *)
-(*            Maxence Guesdon, projet Cristal, INRIA Rocquencourt      *)
-(*                                                                     *)
-(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
-(*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the Q Public License version 1.0.               *)
-(*                                                                     *)
-(***********************************************************************)
+(**************************************************************************)
+(*                                                                        *)
+(*                                 OCaml                                  *)
+(*                                                                        *)
+(*             Maxence Guesdon, projet Cristal, INRIA Rocquencourt        *)
+(*                                                                        *)
+(*   Copyright 2001 Institut National de Recherche en Informatique et     *)
+(*     en Automatique.                                                    *)
+(*                                                                        *)
+(*   All rights reserved.  This file is distributed under the terms of    *)
+(*   the GNU Lesser General Public License version 2.1, with the          *)
+(*   special exception on linking described in the file LICENSE.          *)
+(*                                                                        *)
+(**************************************************************************)
 
 (** Cross referencing. *)
 
-module Name = Odoc_name
 open Odoc_module
 open Odoc_class
 open Odoc_extension
@@ -55,24 +57,23 @@ module P_alias =
          Some (Module_type_alias _) -> true
        | _ -> false
       )
-    let p_class c _ = (false, false)
-    let p_class_type ct _ = (false, false)
-    let p_value v _ = false
+    let p_class _ _ = (false, false)
+    let p_class_type _ _ = (false, false)
+    let p_value _ _ = false
     let p_recfield _ _ _ = false
     let p_const _ _ _ = false
-    let p_type t _ = (false, false)
+    let p_type _ _ = (false, false)
     let p_extension x _ = x.xt_alias <> None
     let p_exception e _ = e.ex_alias <> None
-    let p_attribute a _ = false
-    let p_method m _ = false
-    let p_section s _ = false
+    let p_attribute _ _ = false
+    let p_method _ _ = false
+    let p_section _ _ = false
   end
 
 (** The module used to get the aliased elements. *)
 module Search_alias = Odoc_search.Search (P_alias)
 
 type alias_state =
-    Alias_resolved
   | Alias_to_resolve
 
 (** Couples of module name aliases. *)
@@ -137,36 +138,6 @@ let get_alias_names module_list =
   Hashtbl.clear exception_aliases;
   build_alias_list (Search_alias.search module_list 0)
 
-exception Found of string
-let name_alias =
-  let rec f t name =
-    try
-      match Hashtbl.find t name with
-        (s, Alias_resolved) -> s
-      | (s, Alias_to_resolve) -> f t s
-    with
-      Not_found ->
-        try
-          Hashtbl.iter
-            (fun n2 (n3, _) ->
-              if Name.prefix n2 name then
-                let ln2 = String.length n2 in
-                let s = n3^(String.sub name ln2 ((String.length name) - ln2)) in
-                raise (Found s)
-            )
-            t ;
-          Hashtbl.replace t name (name, Alias_resolved);
-          name
-        with
-          Found s ->
-            let s2 = f t s in
-            Hashtbl.replace t s2 (s2, Alias_resolved);
-            s2
-  in
-  fun name alias_tbl ->
-    f alias_tbl name
-
-
 module Map_ord =
   struct
     type t = string
@@ -185,7 +156,7 @@ let add_known_element name k =
     Not_found ->
       known_elements := Ele_map.add name [k] !known_elements
 
-let rec get_known_elements name =
+let get_known_elements name =
   try Ele_map.find name !known_elements
   with Not_found -> []
 
@@ -319,11 +290,9 @@ let init_known_elements_map module_list =
 
 (** The type to describe the names not found. *)
 type not_found_name =
-    NF_m of Name.t
   | NF_mt of Name.t
   | NF_mmt of Name.t
   | NF_c of Name.t
-  | NF_ct of Name.t
   | NF_cct of Name.t
   | NF_xt of Name.t
   | NF_ex of Name.t
@@ -389,7 +358,7 @@ let rec associate_in_module module_list (acc_b_modif, acc_incomplete_top_module_
      | Module_typeof _ ->
         (acc_b, acc_inc, acc_names)
 
-     | Module_unpack (code, mta) ->
+     | Module_unpack (_code, mta) ->
         begin
           match mta.mta_module with
             Some _ ->
@@ -465,7 +434,8 @@ and associate_in_module_type module_list (acc_b_modif, acc_incomplete_top_module
 and associate_in_module_element module_list m_name (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) element =
    match element with
      Element_module m -> associate_in_module module_list (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) m
-   | Element_module_type mt -> associate_in_module_type module_list (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) mt
+   | Element_module_type mt ->
+       associate_in_module_type module_list (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) mt
    | Element_included_module im ->
        (
         match im.im_module with
@@ -492,9 +462,11 @@ and associate_in_module_element module_list m_name (acc_b_modif, acc_incomplete_
                 (true, acc_incomplete_top_module_names, acc_names_not_found)
        )
    | Element_class cl -> associate_in_class module_list (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) cl
-   | Element_class_type ct -> associate_in_class_type module_list (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) ct
+   | Element_class_type ct ->
+       associate_in_class_type module_list (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) ct
    | Element_value _ -> (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found)
-   | Element_type_extension te -> associate_in_type_extension module_list (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) te
+   | Element_type_extension te ->
+       associate_in_type_extension module_list (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) te
    | Element_exception ex ->
        (
         match ex.ex_alias with
@@ -509,7 +481,9 @@ and associate_in_module_element module_list m_name (acc_b_modif, acc_incomplete_
                   with Not_found -> None
                 in
                 match ex_opt with
-                  None -> (acc_b_modif, (Name.head m_name) :: acc_incomplete_top_module_names, (NF_ex ea.ea_name) :: acc_names_not_found)
+                  None -> (acc_b_modif,
+                           (Name.head m_name) :: acc_incomplete_top_module_names,
+                           (NF_ex ea.ea_name) :: acc_names_not_found)
                 | Some e ->
                     ea.ea_ex <- Some e ;
                     (true, acc_incomplete_top_module_names, acc_names_not_found)
@@ -600,8 +574,8 @@ and associate_in_class module_list (acc_b_modif, acc_incomplete_top_module_names
   in
   iter_kind (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) c.cl_kind
 
-and associate_in_class_type module_list (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) ct =
-  let rec iter_kind (acc_b, acc_inc, acc_names) k =
+and associate_in_class_type _module_list (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) ct =
+  let iter_kind (acc_b, acc_inc, acc_names) k =
     match k with
       Class_signature (inher_l, _) ->
         let f (acc_b2, acc_inc2, acc_names2) ic =
@@ -646,7 +620,7 @@ and associate_in_class_type module_list (acc_b_modif, acc_incomplete_top_module_
   in
   iter_kind (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) ct.clt_kind
 
-and associate_in_type_extension module_list (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) te =
+and associate_in_type_extension _module_list (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) te =
   List.fold_left
     (fun (acc_b_modif, acc_incomplete_top_module_names, acc_names_not_found) xt ->
        match xt.xt_alias with
@@ -661,7 +635,9 @@ and associate_in_type_extension module_list (acc_b_modif, acc_incomplete_top_mod
                      with Not_found -> None
                    in
                      match xt_opt with
-                         None -> (acc_b_modif, (Name.head xt.xt_name) :: acc_incomplete_top_module_names, (NF_xt xa.xa_name) :: acc_names_not_found)
+                         None -> (acc_b_modif,
+                                  (Name.head xt.xt_name) :: acc_incomplete_top_module_names,
+                                  (NF_xt xa.xa_name) :: acc_names_not_found)
                        | Some x ->
                            xa.xa_xt <- Some x ;
                            (true, acc_incomplete_top_module_names, acc_names_not_found))
@@ -691,10 +667,71 @@ let not_found_of_kind kind name =
   | RK_const -> Odoc_messages.cross_const_not_found
   ) name
 
+let query module_list name =
+   match get_known_elements name with
+     | [] ->
+         (
+         try
+           let re = Str.regexp ("^"^(Str.quote name)^"$") in
+            let t = Odoc_search.find_section module_list re in
+            let v2 = (name, Some (RK_section t)) in
+            add_verified v2 ;
+            (name, Some (RK_section t))
+          with
+            Not_found ->
+              (name, None)
+         )
+     | ele :: _ ->
+        (* we look for the first element with this name *)
+        let (name, kind) =
+          match ele with
+            Odoc_search.Res_module m -> (m.m_name, RK_module)
+          | Odoc_search.Res_module_type mt -> (mt.mt_name, RK_module_type)
+          | Odoc_search.Res_class c -> (c.cl_name, RK_class)
+          | Odoc_search.Res_class_type ct -> (ct.clt_name, RK_class_type)
+          | Odoc_search.Res_value v -> (v.val_name, RK_value)
+          | Odoc_search.Res_type t -> (t.ty_name, RK_type)
+          | Odoc_search.Res_extension x -> (x.xt_name, RK_extension)
+          | Odoc_search.Res_exception e -> (e.ex_name, RK_exception)
+          | Odoc_search.Res_attribute a -> (a.att_value.val_name, RK_attribute)
+          | Odoc_search.Res_method m -> (m.met_value.val_name, RK_method)
+          | Odoc_search.Res_section _-> assert false
+          | Odoc_search.Res_recfield (t, f) ->
+              (Printf.sprintf "%s.%s" t.ty_name f.rf_name, RK_recfield)
+          | Odoc_search.Res_const (t, f) ->
+              (Printf.sprintf "%s.%s" t.ty_name f.vc_name, RK_const)
+        in
+        add_verified (name, Some kind) ;
+        (name, Some kind)
+
+
+let rec search_within_ancestry
+    (finalize,initial_name,query as param) ?parent_name name =
+  let name = Odoc_name.normalize_name name in
+  let res = query name in
+  match res with
+  | (name, Some k) -> finalize (Some (name,k))
+  | (_, None) ->
+      match parent_name with
+      | None ->
+          finalize None
+      (* *)
+      | Some p ->
+          let parent_name =
+            match Name.father p with
+              "" -> None
+            | s -> Some s
+          in
+          search_within_ancestry param
+            ?parent_name (Name.concat p initial_name)
+
+let search_within_ancestry finalize query ?parent_name name =
+  search_within_ancestry (finalize, name, query) ?parent_name name
+
+
 let rec assoc_comments_text_elements parent_name module_list t_ele =
   match t_ele with
   | Raw _
-  | Code _
   | CodePre _
   | Latex _
   | Verbatim _ -> t_ele
@@ -713,63 +750,65 @@ let rec assoc_comments_text_elements parent_name module_list t_ele =
   | Title (n, l_opt, t) -> Title (n, l_opt, (assoc_comments_text parent_name module_list t))
   | Link (s, t) -> Link (s, (assoc_comments_text parent_name module_list t))
   | Ref (initial_name, None, text_option) ->
-      (
-       let rec iter_parent ?parent_name name =
-         let name = Odoc_name.normalize_name name in
-         let res =
-           match get_known_elements name with
-             [] ->
-               (
-                try
-                  let re = Str.regexp ("^"^(Str.quote name)^"$") in
-                  let t = Odoc_search.find_section module_list re in
-                  let v2 = (name, Some (RK_section t)) in
-                  add_verified v2 ;
-                  (name, Some (RK_section t))
-              with
-                  Not_found ->
-                    (name, None)
-               )
-           | ele :: _ ->
-           (* we look for the first element with this name *)
-               let (name, kind) =
-                 match ele with
-                   Odoc_search.Res_module m -> (m.m_name, RK_module)
-                 | Odoc_search.Res_module_type mt -> (mt.mt_name, RK_module_type)
-                 | Odoc_search.Res_class c -> (c.cl_name, RK_class)
-                 | Odoc_search.Res_class_type ct -> (ct.clt_name, RK_class_type)
-                 | Odoc_search.Res_value v -> (v.val_name, RK_value)
-                 | Odoc_search.Res_type t -> (t.ty_name, RK_type)
-                 | Odoc_search.Res_extension x -> (x.xt_name, RK_extension)
-                 | Odoc_search.Res_exception e -> (e.ex_name, RK_exception)
-                 | Odoc_search.Res_attribute a -> (a.att_value.val_name, RK_attribute)
-                 | Odoc_search.Res_method m -> (m.met_value.val_name, RK_method)
-                 | Odoc_search.Res_section (_ ,t)-> assert false
-                 | Odoc_search.Res_recfield (t, f) ->
-                     (Printf.sprintf "%s.%s" t.ty_name f.rf_name, RK_recfield)
-                 | Odoc_search.Res_const (t, f) ->
-                     (Printf.sprintf "%s.%s" t.ty_name f.vc_name, RK_const)
-               in
-               add_verified (name, Some kind) ;
-               (name, Some kind)
-         in
-         match res with
-         | (name, Some k) -> Ref (name, Some k, text_option)
-         | (_, None) ->
-             match parent_name with
-               None ->
-                 Odoc_global.pwarning (Odoc_messages.cross_element_not_found initial_name);
-                 Ref (initial_name, None, text_option)
-             | Some p ->
-                 let parent_name =
-                   match Name.father p with
-                     "" -> None
-                   | s -> Some s
-                 in
-                 iter_parent ?parent_name (Name.concat p initial_name)
-       in
-       iter_parent ~parent_name initial_name
-      )
+      let finalize = function
+        | Some (name,k) -> Ref (name, Some k, text_option)
+        | None ->
+            Odoc_global.pwarning
+              (Odoc_messages.cross_element_not_found initial_name);
+            Ref (initial_name, None, text_option) in
+      search_within_ancestry finalize (query module_list) ~parent_name initial_name
+  | Code s ->
+      if not !Odoc_global.show_missed_crossref then
+        t_ele
+      else (* Check if s could be turned into a valid cross-reference *)
+      let name = String.trim s in
+      begin
+        (* First, we ignore code fragments with more than one space-separated
+           words: "word1 word2" *)
+        try  (ignore (String.index name ' '); t_ele)
+        with Not_found ->
+          if name = "" then t_ele
+          else
+            let first_char = name.[0] in
+            (* Then, we only consider code fragments which start with a
+               distinctly uppercase letter *)
+            if Char.uppercase_ascii first_char <> first_char ||
+               Char.lowercase_ascii first_char = first_char then
+              t_ele
+            else
+              (* Some path analysis auxiliary functions *)
+              let path s =
+                String.split_on_char '.' s
+              in
+              let filter =
+                List.filter
+                  (fun s -> s <> "" && s.[0] = Char.uppercase_ascii s.[0]) in
+              let rec is_prefix prefix full =
+                match prefix, full with
+                | [], _ -> true
+                | a :: pre, b :: f when a = b -> is_prefix pre f
+                | _ -> false in
+              let p = filter @@ path name and parent_p = path parent_name in
+              let is_path_suffix () =
+                is_prefix (List.rev @@ p) (List.rev @@ parent_p ) in
+              (* heuristic:
+                 - if name = parent_name: we are using the name of an element
+                 or module in its definition, no need of cross_reference
+                 - if the path of name is a suffix of the parent path, we
+                 are in the same module, maybe the same function. To decreace
+                 the false positive rate, we stop here *)
+              if name = parent_name || is_path_suffix () then
+                t_ele
+              else
+                let finalize = function
+                  | None -> t_ele
+                  | Some _ ->
+                      Odoc_global.pwarning @@
+                      Odoc_messages.code_could_be_cross_reference name parent_name;
+                      t_ele in
+                search_within_ancestry finalize (query module_list) ~parent_name
+                  name
+      end
   | Ref (initial_name, Some kind, text_option) ->
       (
        let rec iter_parent ?parent_name name =
@@ -781,7 +820,7 @@ let rec assoc_comments_text_elements parent_name module_list t_ele =
              match kind with
              | RK_section _ ->
                  (
-                  (** we just verify that we find an element of this kind with this name *)
+                  (* we just verify that we find an element of this kind with this name *)
                   try
                     let re = Str.regexp ("^"^(Str.quote name)^"$") in
                     let t = Odoc_search.find_section module_list re in
@@ -983,7 +1022,7 @@ and assoc_comments_parameter parent_name module_list p =
   match p with
     Simple_name sn ->
       sn.sn_text <- ao (assoc_comments_text parent_name module_list) sn.sn_text
-  | Tuple (l, t) ->
+  | Tuple (l, _) ->
       List.iter (assoc_comments_parameter parent_name module_list) l
 
 and assoc_comments_parameter_list parent_name module_list pl =
@@ -1079,11 +1118,9 @@ let associate module_list =
            Odoc_global.pwarning
              (
               match nf with
-                NF_m n -> Odoc_messages.cross_module_not_found n
               | NF_mt n -> Odoc_messages.cross_module_type_not_found n
               | NF_mmt n -> Odoc_messages.cross_module_or_module_type_not_found n
               | NF_c n -> Odoc_messages.cross_class_not_found n
-              | NF_ct n -> Odoc_messages.cross_class_type_not_found n
               | NF_cct n -> Odoc_messages.cross_class_or_class_type_not_found n
               | NF_xt n -> Odoc_messages.cross_extension_not_found n
               | NF_ex n -> Odoc_messages.cross_exception_not_found n
