@@ -1,14 +1,17 @@
-(***********************************************************************)
-(*                                                                     *)
-(*                             OCamldoc                                *)
-(*                                                                     *)
-(*            Maxence Guesdon, projet Cristal, INRIA Rocquencourt      *)
-(*                                                                     *)
-(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
-(*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the Q Public License version 1.0.               *)
-(*                                                                     *)
-(***********************************************************************)
+(**************************************************************************)
+(*                                                                        *)
+(*                                 OCaml                                  *)
+(*                                                                        *)
+(*             Maxence Guesdon, projet Cristal, INRIA Rocquencourt        *)
+(*                                                                        *)
+(*   Copyright 2001 Institut National de Recherche en Informatique et     *)
+(*     en Automatique.                                                    *)
+(*                                                                        *)
+(*   All rights reserved.  This file is distributed under the terms of    *)
+(*   the GNU Lesser General Public License version 2.1, with the          *)
+(*   special exception on linking described in the file LICENSE.          *)
+(*                                                                        *)
+(**************************************************************************)
 
 (** Representation and manipulation of modules and module types. *)
 
@@ -216,61 +219,12 @@ let included_modules l =
     []
     l
 
-(** Returns the list of elements of a module.
-   @param trans indicates if, for aliased modules, we must perform a transitive search.*)
-let rec module_elements ?(trans=true) m =
-  let rec iter_kind = function
-      Module_struct l ->
-        print_DEBUG "Odoc_module.module_element: Module_struct";
-        l
-    | Module_alias ma ->
-        print_DEBUG "Odoc_module.module_element: Module_alias";
-        if trans then
-          match ma.ma_module with
-            None -> []
-          | Some (Mod m) -> module_elements m
-          | Some (Modtype mt) -> module_type_elements mt
-        else
-          []
-    | Module_functor (_, k)
-    | Module_apply (k, _) ->
-        print_DEBUG "Odoc_module.module_element: Module_functor ou Module_apply";
-        iter_kind k
-    | Module_with (tk,_) ->
-        print_DEBUG "Odoc_module.module_element: Module_with";
-        module_type_elements ~trans: trans
-          { mt_name = "" ; mt_info = None ; mt_type = None ;
-            mt_is_interface = false ; mt_file = "" ; mt_kind = Some tk ;
-            mt_loc = Odoc_types.dummy_loc ;
-          }
-    | Module_constraint (k, tk) ->
-        print_DEBUG "Odoc_module.module_element: Module_constraint";
-      (* A VOIR : utiliser k ou tk ? *)
-        module_elements ~trans: trans
-          { m_name = "" ;
-            m_info = None ;
-            m_type = Types.Mty_signature [] ;
-            m_is_interface = false ; m_file = "" ; m_kind = k ;
-            m_loc = Odoc_types.dummy_loc ;
-            m_top_deps = [] ;
-            m_code = None ;
-            m_code_intf = None ;
-            m_text_only = false ;
-          }
-    | Module_typeof _ -> []
-    | Module_unpack _ -> []
-(*
-   module_type_elements ~trans: trans
-   { mt_name = "" ; mt_info = None ; mt_type = None ;
-   mt_is_interface = false ; mt_file = "" ; mt_kind = Some tk ;
-   mt_loc = Odoc_types.dummy_loc }
-*)
-  in
-  iter_kind m.m_kind
+module S = Misc.StringSet
+
 
 (** Returns the list of elements of a module type.
    @param trans indicates if, for aliased modules, we must perform a transitive search.*)
-and module_type_elements ?(trans=true) mt =
+let rec module_type_elements ?(trans=true) mt =
   let rec iter_kind = function
     | None -> []
     | Some (Module_type_struct l) -> l
@@ -291,6 +245,68 @@ and module_type_elements ?(trans=true) mt =
   in
   iter_kind mt.mt_kind
 
+(** Returns the list of elements of a module.
+   @param trans indicates if, for aliased modules, we must perform a transitive search.
+*)
+let module_elements ?(trans=true) m =
+(* visited is used to guard against aliases loop
+     (e.g [module rec M:sig end=M] induced loop.
+*)
+  let rec module_elements visited ?(trans=true) m =
+    let rec iter_kind = function
+        Module_struct l ->
+          print_DEBUG "Odoc_module.module_elements: Module_struct";
+          l
+      | Module_alias ma ->
+          print_DEBUG "Odoc_module.module_elements: Module_alias";
+          if trans then
+            match ma.ma_module with
+              None -> []
+            | Some (Mod m') ->
+                if S.mem m'.m_name visited then
+                  []
+                else
+                  module_elements (S.add m'.m_name visited) m'
+            | Some (Modtype mt) -> module_type_elements mt
+          else
+            []
+      | Module_functor (_, k)
+      | Module_apply (k, _) ->
+          print_DEBUG "Odoc_module.module_elements: Module_functor ou Module_apply";
+          iter_kind k
+      | Module_with (tk,_) ->
+          print_DEBUG "Odoc_module.module_elements: Module_with";
+          module_type_elements ~trans: trans
+            { mt_name = "" ; mt_info = None ; mt_type = None ;
+              mt_is_interface = false ; mt_file = "" ; mt_kind = Some tk ;
+              mt_loc = Odoc_types.dummy_loc ;
+            }
+      | Module_constraint (k, _tk) ->
+          print_DEBUG "Odoc_module.module_elements: Module_constraint";
+          (* FIXME : use k or tk ? *)
+          module_elements visited ~trans: trans
+            { m_name = "" ;
+              m_info = None ;
+              m_type = Types.Mty_signature [] ;
+              m_is_interface = false ; m_file = "" ; m_kind = k ;
+              m_loc = Odoc_types.dummy_loc ;
+              m_top_deps = [] ;
+              m_code = None ;
+              m_code_intf = None ;
+              m_text_only = false ;
+            }
+      | Module_typeof _ -> []
+      | Module_unpack _ -> []
+(*
+   module_type_elements ~trans: trans
+   { mt_name = "" ; mt_info = None ; mt_type = None ;
+   mt_is_interface = false ; mt_file = "" ; mt_kind = Some tk ;
+   mt_loc = Odoc_types.dummy_loc }
+*)
+    in
+    iter_kind m.m_kind in
+  module_elements S.empty ~trans m
+
 (** Returns the list of values of a module.
   @param trans indicates if, for aliased modules, we must perform a transitive search.*)
 let module_values ?(trans=true) m = values (module_elements ~trans m)
@@ -413,7 +429,7 @@ and module_parameters ?(trans=true) m =
           | Some (Modtype mt) -> module_type_parameters ~trans mt
         else
           []
-    | Module_constraint (k, tk) ->
+    | Module_constraint (_k, tk) ->
         module_type_parameters ~trans: trans
           { mt_name = "" ; mt_info = None ; mt_type = None ;
             mt_is_interface = false ; mt_file = "" ; mt_kind = Some tk ;
@@ -456,20 +472,22 @@ let rec module_type_is_functor mt =
 
 (** The module is a functor if is defined as a functor or if it is an alias for a functor. *)
 let module_is_functor m =
-  let rec iter = function
+  let rec iter visited = function
       Module_functor _ -> true
     | Module_alias ma ->
         (
-         match ma.ma_module with
-           None -> false
-         | Some (Mod mo) -> iter mo.m_kind
-         | Some (Modtype mt) -> module_type_is_functor mt
+          not (S.mem ma.ma_name visited)
+          &&
+          match ma.ma_module with
+            None -> false
+          | Some (Mod mo) -> iter (S.add ma.ma_name visited) mo.m_kind
+          | Some (Modtype mt) -> module_type_is_functor mt
         )
     | Module_constraint (k, _) ->
-        iter k
+        iter visited k
     | _ -> false
   in
-  iter m.m_kind
+  iter S.empty m.m_kind
 
 (** Returns the list of values of a module type.
   @param trans indicates if, for aliased modules, we must perform a transitive search.*)
