@@ -1,15 +1,21 @@
-(***********************************************************************)
-(*                                                                     *)
-(*                                OCaml                                *)
-(*                                                                     *)
-(*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
-(*                                                                     *)
-(*  Copyright 1996 Institut National de Recherche en Informatique et   *)
-(*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License, with    *)
-(*  the special exception on linking described in file ../../LICENSE.  *)
-(*                                                                     *)
-(***********************************************************************)
+(**************************************************************************)
+(*                                                                        *)
+(*                                 OCaml                                  *)
+(*                                                                        *)
+(*             Xavier Leroy, projet Cristal, INRIA Rocquencourt           *)
+(*                                                                        *)
+(*   Copyright 1996 Institut National de Recherche en Informatique et     *)
+(*     en Automatique.                                                    *)
+(*                                                                        *)
+(*   All rights reserved.  This file is distributed under the terms of    *)
+(*   the GNU Lesser General Public License version 2.1, with the          *)
+(*   special exception on linking described in the file LICENSE.          *)
+(*                                                                        *)
+(**************************************************************************)
+
+(* In this module, [@ocaml.warning "-3"] is used in several places
+   that use deprecated functions to preserve legacy behavior.
+   It overrides -w @3 given on the command line. *)
 
 (** String utilities *)
 
@@ -88,9 +94,9 @@ module Charset =
       r
 
     let fold_case s =
-      let r = make_empty() in
-      iter (fun c -> add r (Char.lowercase c); add r (Char.uppercase c)) s;
-      r
+      (let r = make_empty() in
+       iter (fun c -> add r (Char.lowercase c); add r (Char.uppercase c)) s;
+       r)[@ocaml.warning "-3"]
 
   end
 
@@ -157,16 +163,16 @@ let displ dest from = dest - from - 1
 (* Determine if a regexp can match the empty string *)
 
 let rec is_nullable = function
-    Char c -> false
+    Char _ -> false
   | String s -> s = ""
-  | CharClass(cl, cmpl) -> false
+  | CharClass _ -> false
   | Seq rl -> List.for_all is_nullable rl
   | Alt (r1, r2) -> is_nullable r1 || is_nullable r2
-  | Star r -> true
+  | Star _ -> true
   | Plus r -> is_nullable r
-  | Option r -> true
-  | Group(n, r) -> is_nullable r
-  | Refgroup n -> true
+  | Option _ -> true
+  | Group(_, r) -> is_nullable r
+  | Refgroup _ -> true
   | Bol -> true
   | Eol -> true
   | Wordboundary -> true
@@ -181,11 +187,11 @@ let rec first = function
   | CharClass(cl, cmpl) -> if cmpl then Charset.complement cl else cl
   | Seq rl -> first_seq rl
   | Alt (r1, r2) -> Charset.union (first r1) (first r2)
-  | Star r -> Charset.full
+  | Star _ -> Charset.full
   | Plus r -> first r
-  | Option r -> Charset.full
-  | Group(n, r) -> first r
-  | Refgroup n -> Charset.full
+  | Option _ -> Charset.full
+  | Group(_, r) -> first r
+  | Refgroup _ -> Charset.full
   | Bol -> Charset.full
   | Eol -> Charset.full
   | Wordboundary -> Charset.full
@@ -195,7 +201,7 @@ and first_seq = function
   | (Bol | Eol | Wordboundary) :: rl -> first_seq rl
   | Star r :: rl -> Charset.union (first r) (first_seq rl)
   | Option r :: rl -> Charset.union (first r) (first_seq rl)
-  | r :: rl -> first r
+  | r :: _ -> first r
 
 (* Transform a Char or CharClass regexp into a character class *)
 
@@ -211,9 +217,9 @@ let charclass_of_regexp fold_case re =
 (* The case fold table: maps characters to their lowercase equivalent *)
 
 let fold_case_table =
-  let t = Bytes.create 256 in
-  for i = 0 to 255 do Bytes.set t i (Char.lowercase(Char.chr i)) done;
-  Bytes.to_string t
+  (let t = Bytes.create 256 in
+   for i = 0 to 255 do Bytes.set t i (Char.lowercase(Char.chr i)) done;
+   Bytes.to_string t)[@ocaml.warning "-3"]
 
 module StringMap =
   Map.Make(struct type t = string let compare (x:t) y = compare x y end)
@@ -242,7 +248,7 @@ let compile fold_case re =
     incr progpos in
   (* Reserve an instruction slot and return its position *)
   let emit_hole () =
-    let p = !progpos in incr progpos; p in
+    let p = !progpos in emit_instr op_CHAR 0; p in
   (* Fill a reserved instruction slot with a GOTO or PUSHBACK instruction *)
   let patch_instr pos opc dest =
     (!prog).(pos) <- (instr opc (displ dest pos)) in
@@ -270,6 +276,7 @@ let compile fold_case re =
     Char c ->
       if fold_case then
         emit_instr op_CHARNORM (Char.code (Char.lowercase c))
+          [@ocaml.warning "-3"]
       else
         emit_instr op_CHAR (Char.code c)
   | String s ->
@@ -278,6 +285,7 @@ let compile fold_case re =
       | 1 ->
         if fold_case then
           emit_instr op_CHARNORM (Char.code (Char.lowercase s.[0]))
+            [@ocaml.warning "-3"]
         else
           emit_instr op_CHAR (Char.code s.[0])
       | _ ->
@@ -291,6 +299,7 @@ let compile fold_case re =
         with Not_found ->
           if fold_case then
             emit_instr op_STRINGNORM (cpool_index (String.lowercase s))
+              [@ocaml.warning "-3"]
           else
             emit_instr op_STRING (cpool_index s)
       end
@@ -372,13 +381,13 @@ let compile fold_case re =
       let lbl = !progpos in
       patch_instr pos_pushback op_PUSHBACK lbl
   | Group(n, r) ->
-      if n >= 32 then failwith "too many \\(...\\) groups";
       emit_instr op_BEGGROUP n;
       emit_code r;
       emit_instr op_ENDGROUP n;
       numgroups := max !numgroups (n+1)
   | Refgroup n ->
-      emit_instr op_REFGROUP n
+      emit_instr op_REFGROUP n;
+      numgroups := max !numgroups (n+1)
   | Bol ->
       emit_instr op_BOL 0
   | Eol ->
@@ -511,12 +520,10 @@ let parse s =
           assert false
       | '(' ->
           let group_no = !group_counter in
-          if group_no < 32 then incr group_counter;
+          incr group_counter;
           let (r, j) = regexp0 (i+1) in
           if j + 1 < len && s.[j] = '\\' && s.[j+1] = ')' then
-            if group_no < 32
-            then (Group(group_no, r), j + 2)
-            else (r, j + 2)
+            (Group(group_no, r), j + 2)
           else
             failwith "\\( group not closed by \\)"
       | '1' .. '9' as c ->
