@@ -1,15 +1,19 @@
-/***********************************************************************/
-/*                                                                     */
-/*                                OCaml                                */
-/*                                                                     */
-/*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         */
-/*                                                                     */
-/*  Copyright 1996 Institut National de Recherche en Informatique et   */
-/*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License, with    */
-/*  the special exception on linking described in file ../../LICENSE.  */
-/*                                                                     */
-/***********************************************************************/
+/**************************************************************************/
+/*                                                                        */
+/*                                 OCaml                                  */
+/*                                                                        */
+/*             Xavier Leroy, projet Cristal, INRIA Rocquencourt           */
+/*                                                                        */
+/*   Copyright 1996 Institut National de Recherche en Informatique et     */
+/*     en Automatique.                                                    */
+/*                                                                        */
+/*   All rights reserved.  This file is distributed under the terms of    */
+/*   the GNU Lesser General Public License version 2.1, with the          */
+/*   special exception on linking described in the file LICENSE.          */
+/*                                                                        */
+/**************************************************************************/
+
+#define CAML_INTERNALS
 
 /* The thread scheduler */
 
@@ -80,7 +84,7 @@ struct caml_thread_struct {
   value * sp;
   value * trapsp;
   value backtrace_pos;          /* The backtrace info for this thread */
-  code_t * backtrace_buffer;
+  backtrace_slot * backtrace_buffer;
   value backtrace_last_exn;
   value status;                 /* RUNNABLE, KILLED. etc (see below) */
   value fd;     /* File descriptor on which we're doing read or write */
@@ -124,7 +128,7 @@ static caml_thread_t curr_thread = NULL;
 /* Identifier for next thread creation */
 static value next_ident = Val_int(0);
 
-#define Assign(dst,src) modify((value *)&(dst), (value)(src))
+#define Assign(dst,src) caml_modify((value *)&(dst), (value)(src))
 
 /* Scan the stacks of the other threads */
 
@@ -140,7 +144,7 @@ static void thread_scan_roots(scanning_action action)
   /* Don't scan curr_thread->sp, this has already been done.
      Don't scan local roots either, for the same reason. */
   for (th = start->next; th != start; th = th->next) {
-    do_local_roots(action, th->sp, th->stack_high, NULL);
+    caml_do_local_roots(action, th->sp, th->stack_high, NULL);
   }
   /* Hook */
   if (prev_scan_roots_hook != NULL) (*prev_scan_roots_hook)(action);
@@ -159,20 +163,20 @@ value thread_initialize(value unit)       /* ML */
   if (curr_thread != NULL) return Val_unit;
   /* Create a descriptor for the current thread */
   curr_thread =
-    (caml_thread_t) alloc_shr(sizeof(struct caml_thread_struct)
+    (caml_thread_t) caml_alloc_shr(sizeof(struct caml_thread_struct)
                               / sizeof(value), 0);
   curr_thread->ident = next_ident;
   next_ident = Val_int(Int_val(next_ident) + 1);
   curr_thread->next = curr_thread;
   curr_thread->prev = curr_thread;
-  curr_thread->stack_low = stack_low;
-  curr_thread->stack_high = stack_high;
-  curr_thread->stack_threshold = stack_threshold;
-  curr_thread->sp = extern_sp;
-  curr_thread->trapsp = trapsp;
-  curr_thread->backtrace_pos = Val_int(backtrace_pos);
-  curr_thread->backtrace_buffer = backtrace_buffer;
-  caml_initialize (&curr_thread->backtrace_last_exn, backtrace_last_exn);
+  curr_thread->stack_low = caml_stack_low;
+  curr_thread->stack_high = caml_stack_high;
+  curr_thread->stack_threshold = caml_stack_threshold;
+  curr_thread->sp = caml_extern_sp;
+  curr_thread->trapsp = caml_trapsp;
+  curr_thread->backtrace_pos = Val_int(caml_backtrace_pos);
+  curr_thread->backtrace_buffer = caml_backtrace_buffer;
+  caml_initialize (&curr_thread->backtrace_last_exn, caml_backtrace_last_exn);
   curr_thread->status = RUNNABLE;
   curr_thread->fd = Val_int(0);
   curr_thread->readfds = NO_FDS;
@@ -183,8 +187,8 @@ value thread_initialize(value unit)       /* ML */
   curr_thread->waitpid = NO_WAITPID;
   curr_thread->retval = Val_unit;
   /* Initialize GC */
-  prev_scan_roots_hook = scan_roots_hook;
-  scan_roots_hook = thread_scan_roots;
+  prev_scan_roots_hook = caml_scan_roots_hook;
+  caml_scan_roots_hook = thread_scan_roots;
   /* Set standard file descriptors to non-blocking mode */
   stdin_initial_status = fcntl(0, F_GETFL);
   stdout_initial_status = fcntl(1, F_GETFL);
@@ -220,7 +224,7 @@ value thread_new(value clos)          /* ML */
   caml_thread_t th;
   /* Allocate the thread and its stack */
   Begin_root(clos);
-    th = (caml_thread_t) alloc_shr(sizeof(struct caml_thread_struct)
+    th = (caml_thread_t) caml_alloc_shr(sizeof(struct caml_thread_struct)
                                    / sizeof(value), 0);
   End_roots();
   th->ident = next_ident;
@@ -299,17 +303,17 @@ static value schedule_thread(void)
   int need_select, need_wait;
 
   /* Don't allow preemption during a callback */
-  if (callback_depth > 1) return curr_thread->retval;
+  if (caml_callback_depth > 1) return curr_thread->retval;
 
   /* Save the status of the current thread */
-  curr_thread->stack_low = stack_low;
-  curr_thread->stack_high = stack_high;
-  curr_thread->stack_threshold = stack_threshold;
-  curr_thread->sp = extern_sp;
-  curr_thread->trapsp = trapsp;
-  curr_thread->backtrace_pos = Val_int(backtrace_pos);
-  curr_thread->backtrace_buffer = backtrace_buffer;
-  caml_modify (&curr_thread->backtrace_last_exn, backtrace_last_exn);
+  curr_thread->stack_low = caml_stack_low;
+  curr_thread->stack_high = caml_stack_high;
+  curr_thread->stack_threshold = caml_stack_threshold;
+  curr_thread->sp = caml_extern_sp;
+  curr_thread->trapsp = caml_trapsp;
+  curr_thread->backtrace_pos = Val_int(caml_backtrace_pos);
+  curr_thread->backtrace_buffer = caml_backtrace_buffer;
+  caml_modify (&curr_thread->backtrace_last_exn, caml_backtrace_last_exn);
 
 try_again:
   /* Find if a thread is runnable.
@@ -399,9 +403,9 @@ try_again:
     else {
       delay_ptr = NULL;
     }
-    enter_blocking_section();
+    caml_enter_blocking_section();
     retcode = select(FD_SETSIZE, &readfds, &writefds, &exceptfds, delay_ptr);
-    leave_blocking_section();
+    caml_leave_blocking_section();
     if (retcode == -1)
       switch (errno) {
       case EINTR:
@@ -426,7 +430,7 @@ try_again:
         retcode = FD_SETSIZE;
         break;
       default:
-        sys_error(NO_ARG);
+        caml_sys_error(NO_ARG);
       }
     if (retcode > 0) {
       /* Some descriptors are ready.
@@ -458,7 +462,7 @@ try_again:
             w = inter_fdlist_set(th->writefds, &writefds, &retcode);
             e = inter_fdlist_set(th->exceptfds, &exceptfds, &retcode);
             if (r != NO_FDS || w != NO_FDS || e != NO_FDS) {
-              value retval = alloc_small(3, TAG_RESUMED_SELECT);
+              value retval = caml_alloc_small(3, TAG_RESUMED_SELECT);
               Field(retval, 0) = r;
               Field(retval, 1) = w;
               Field(retval, 2) = e;
@@ -483,7 +487,7 @@ try_again:
   }
 
   /* If we haven't something to run at that point, we're in big trouble. */
-  if (run_thread == NULL) invalid_argument("Thread: deadlock");
+  if (run_thread == NULL) caml_invalid_argument("Thread: deadlock");
 
   /* Free everything the thread was waiting on */
   Assign(run_thread->readfds, NO_FDS);
@@ -495,14 +499,14 @@ try_again:
 
   /* Activate the thread */
   curr_thread = run_thread;
-  stack_low = curr_thread->stack_low;
-  stack_high = curr_thread->stack_high;
-  stack_threshold = curr_thread->stack_threshold;
-  extern_sp = curr_thread->sp;
-  trapsp = curr_thread->trapsp;
-  backtrace_pos = Int_val(curr_thread->backtrace_pos);
-  backtrace_buffer = curr_thread->backtrace_buffer;
-  backtrace_last_exn = curr_thread->backtrace_last_exn;
+  caml_stack_low = curr_thread->stack_low;
+  caml_stack_high = curr_thread->stack_high;
+  caml_stack_threshold = curr_thread->stack_threshold;
+  caml_extern_sp = curr_thread->sp;
+  caml_trapsp = curr_thread->trapsp;
+  caml_backtrace_pos = Int_val(curr_thread->backtrace_pos);
+  caml_backtrace_buffer = curr_thread->backtrace_buffer;
+  caml_backtrace_last_exn = curr_thread->backtrace_last_exn;
   return curr_thread->retval;
 }
 
@@ -511,7 +515,7 @@ try_again:
 
 static void check_callback(void)
 {
-  if (callback_depth > 1)
+  if (caml_callback_depth > 1)
     caml_fatal_error("Thread: deadlock during callback");
 }
 
@@ -533,20 +537,20 @@ static void thread_reschedule(void)
   Assert(curr_thread != NULL);
   /* Pop accu from event frame, making it look like a C_CALL frame
      followed by a RETURN frame */
-  accu = *extern_sp++;
+  accu = *caml_extern_sp++;
   /* Reschedule */
   Assign(curr_thread->retval, accu);
   accu = schedule_thread();
   /* Push accu below C_CALL frame so that it looks like an event frame */
-  *--extern_sp = accu;
+  *--caml_extern_sp = accu;
 }
 
 /* Request a re-scheduling as soon as possible */
 
 value thread_request_reschedule(value unit)    /* ML */
 {
-  async_action_hook = thread_reschedule;
-  something_to_do = 1;
+  caml_async_action_hook = thread_reschedule;
+  caml_something_to_do = 1;
   return Val_unit;
 }
 
@@ -570,7 +574,7 @@ static value thread_wait_rw(int kind, value fd)
   if (curr_thread == NULL) return RESUMED_WAKEUP;
   /* As a special case, if we're in a callback, don't fail but block
      the whole process till I/O is possible */
-  if (callback_depth > 1) {
+  if (caml_callback_depth > 1) {
     fd_set fds;
     FD_ZERO(&fds);
     FD_SET(Int_val(fd), &fds);
@@ -605,7 +609,7 @@ static value thread_wait_timed_rw(int kind, value arg)
   check_callback();
   curr_thread->fd = Field(arg, 0);
   date = timeofday() + Double_val(Field(arg, 1));
-  Assign(curr_thread->delay, copy_double(date));
+  Assign(curr_thread->delay, caml_copy_double(date));
   curr_thread->status = kind | BLOCKED_DELAY;
   return schedule_thread();
 }
@@ -632,7 +636,7 @@ value thread_select(value arg)        /* ML */
   date = Double_val(Field(arg, 3));
   if (date >= 0.0) {
     date += timeofday();
-    Assign(curr_thread->delay, copy_double(date));
+    Assign(curr_thread->delay, caml_copy_double(date));
     curr_thread->status = BLOCKED_SELECT | BLOCKED_DELAY;
   } else {
     curr_thread->status = BLOCKED_SELECT;
@@ -672,7 +676,7 @@ value thread_delay(value time)          /* ML */
   Assert(curr_thread != NULL);
   check_callback();
   curr_thread->status = BLOCKED_DELAY;
-  Assign(curr_thread->delay, copy_double(date));
+  Assign(curr_thread->delay, caml_copy_double(date));
   return schedule_thread();
 }
 
@@ -710,9 +714,9 @@ value thread_wakeup(value thread)     /* ML */
     Assign(th->retval, RESUMED_WAKEUP);
     break;
   case KILLED:
-    failwith("Thread.wakeup: killed thread");
+    caml_failwith("Thread.wakeup: killed thread");
   default:
-    failwith("Thread.wakeup: thread not suspended");
+    caml_failwith("Thread.wakeup: thread not suspended");
   }
   return Val_unit;
 }
@@ -731,9 +735,9 @@ value thread_kill(value thread)       /* ML */
 {
   value retval = Val_unit;
   caml_thread_t th = (caml_thread_t) thread;
-  if (th->status == KILLED) failwith("Thread.kill: killed thread");
+  if (th->status == KILLED) caml_failwith("Thread.kill: killed thread");
   /* Don't paint ourselves in a corner */
-  if (th == th->next) failwith("Thread.kill: cannot kill the last thread");
+  if (th == th->next) caml_failwith("Thread.kill: cannot kill the last thread");
   /* This thread is no longer waiting on anything */
   th->status = KILLED;
   /* If this is the current thread, activate another one */
@@ -747,7 +751,7 @@ value thread_kill(value thread)       /* ML */
   Assign(th->prev->next, th->next);
   Assign(th->next->prev, th->prev);
   /* Free its resources */
-  stat_free((char *) th->stack_low);
+  caml_stat_free((char *) th->stack_low);
   th->stack_low = NULL;
   th->stack_high = NULL;
   th->stack_threshold = NULL;
@@ -764,11 +768,11 @@ value thread_kill(value thread)       /* ML */
 
 value thread_uncaught_exception(value exn)  /* ML */
 {
-  char * msg = format_caml_exception(exn);
+  char * msg = caml_format_exception(exn);
   fprintf(stderr, "Thread %d killed on uncaught exception %s\n",
           Int_val(curr_thread->ident), msg);
   free(msg);
-  if (backtrace_active) print_exception_backtrace();
+  if (caml_backtrace_active) caml_print_exception_backtrace();
   fflush(stderr);
   return Val_unit;
 }
@@ -796,7 +800,7 @@ static value inter_fdlist_set(value fdl, fd_set *set, int *count)
     for (res = NO_FDS; fdl != NO_FDS; fdl = Field(fdl, 1)) {
       int fd = Int_val(Field(fdl, 0));
       if (FD_ISSET(fd, set)) {
-        cons = alloc_small(2, 0);
+        cons = caml_alloc_small(2, 0);
         Field(cons, 0) = Val_int(fd);
         Field(cons, 1) = res;
         res = cons;
@@ -845,19 +849,19 @@ static value alloc_process_status(int pid, int status)
   value st, res;
 
   if (WIFEXITED(status)) {
-    st = alloc_small(1, TAG_WEXITED);
+    st = caml_alloc_small(1, TAG_WEXITED);
     Field(st, 0) = Val_int(WEXITSTATUS(status));
   }
   else if (WIFSTOPPED(status)) {
-    st = alloc_small(1, TAG_WSTOPPED);
+    st = caml_alloc_small(1, TAG_WSTOPPED);
     Field(st, 0) = Val_int(WSTOPSIG(status));
   }
   else {
-    st = alloc_small(1, TAG_WSIGNALED);
+    st = caml_alloc_small(1, TAG_WSIGNALED);
     Field(st, 0) = Val_int(WTERMSIG(status));
   }
   Begin_root(st);
-    res = alloc_small(2, TAG_RESUMED_WAIT);
+    res = caml_alloc_small(2, TAG_RESUMED_WAIT);
     Field(res, 0) = Val_int(pid);
     Field(res, 1) = st;
   End_roots();
