@@ -4,6 +4,10 @@ val bind_ident: string -> Ident.t
 val find_ident: string -> Ident.t
 val unbind_ident: Ident.t -> unit
 
+val find_label: string -> int
+
+val debuginfo: ?loc:Location.t -> unit -> Debuginfo.t
+
 type error =
     Unbound of string
 
