@@ -20,11 +20,11 @@
 int fib(int n)
 {
   value * fib_closure = caml_named_value("fib");
-  return Int_val(callback(*fib_closure, Val_int(n)));
+  return Int_val(caml_callback(*fib_closure, Val_int(n)));
 }
 
 char * format_result(int n)
 {
   value * format_result_closure = caml_named_value("format_result");
-  return strdup(String_val(callback(*format_result_closure, Val_int(n))));
+  return strdup(String_val(caml_callback(*format_result_closure, Val_int(n))));
 }
