@@ -1,3 +1,5 @@
+(* Ignore OCAMLRUNPARAM=b to be reproducible *)
+Printexc.record_backtrace false;;
 
 type foo = ..
 ;;
@@ -200,10 +202,12 @@ type +'a foo = ..
 type 'a foo += A of (int -> 'a)
 ;;
 
-type 'a foo += B of ('a -> int) (* ERROR: Parameter variances are not satisfied *)
+type 'a foo += B of ('a -> int)
+    (* ERROR: Parameter variances are not satisfied *)
 ;;
 
-type _ foo += C : ('a -> int) -> 'a foo (* ERROR: Parameter variances are not satisfied *)
+type _ foo += C : ('a -> int) -> 'a foo
+    (* ERROR: Parameter variances are not satisfied *)
 ;;
 
 type 'a bar = ..
@@ -294,19 +298,22 @@ type foo +=
   | Bar of int
 ;;
 
-let n1 = Obj.extension_name Foo
+let extension_name e = Obj.extension_name (Obj.extension_constructor e);;
+let extension_id e = Obj.extension_id (Obj.extension_constructor e);;
+
+let n1 = extension_name Foo
 ;;
 
-let n2 = Obj.extension_name (Bar 1)
+let n2 = extension_name (Bar 1)
 ;;
 
-let t = (Obj.extension_id (Bar 2)) = (Obj.extension_id (Bar 3)) (* true *)
+let t = (extension_id (Bar 2)) = (extension_id (Bar 3)) (* true *)
 ;;
 
-let f = (Obj.extension_id (Bar 2)) = (Obj.extension_id Foo) (* false *)
+let f = (extension_id (Bar 2)) = (extension_id Foo) (* false *)
 ;;
 
-let is_foo x = (Obj.extension_id Foo) = (Obj.extension_id x)
+let is_foo x = (extension_id Foo) = (extension_id x)
 
 type foo += Foo
 ;;
@@ -314,8 +321,8 @@ type foo += Foo
 let f = is_foo Foo
 ;;
 
-let _ = Obj.extension_name 7 (* Invald_arg *)
+let _ = Obj.extension_constructor 7 (* Invald_arg *)
 ;;
 
-let _ = Obj.extension_id (object method m = 3 end) (* Invald_arg *)
+let _ = Obj.extension_constructor (object method m = 3 end) (* Invald_arg *)
 ;;
