@@ -68,3 +68,9 @@ end = struct
   type t += Private_ext
 end
 ;;
+
+module Pr7438 : sig
+end = struct
+  module type S = sig type t = private [> `Foo] end
+  module type X = sig type t = private [> `Foo | `Bar] include S with type t := t end
+end;;
