@@ -51,6 +51,8 @@ let incompatible o =
 module Options = Main_args.Make_optcomp_options (struct
   let _a () = make_archive := true; option "-a" ()
   let _absname = option "-absname"
+  let _afl_instrument = option "-afl-instrument"
+  let _afl_inst_ratio n = option_with_int "-afl-inst-ratio" n
   let _annot = option "-annot"
   let _binannot = option "-bin-annot"
   let _c = option "-c"
@@ -172,6 +174,8 @@ module Options = Main_args.Make_optcomp_options (struct
   let _dtimings = option "-dtimings"
   let _opaque = option "-opaque"
 
+  let _args = Arg.read_arg
+  let _args0 = Arg.read_arg0
   let anonymous = process_file
 end);;
 
@@ -190,7 +194,7 @@ let optlist =
         \032     t  try ... with")
     :: Options.list
 in
-Arg.parse optlist process_file usage;
+Arg.parse_expand optlist process_file usage;
 if !with_impl && !with_intf then begin
   fprintf stderr "ocamloptp cannot deal with both \"-impl\" and \"-intf\"\n";
   fprintf stderr "please compile interfaces and implementations separately\n";
