@@ -151,15 +151,15 @@ module Make(O : OBJ)(EVP : EVALPATH with type valu = O.t) = struct
                 (fun x -> Oval_int64 (O.obj x : int64)) ))
     ] : (Path.t * printer) list)
 
-    let exn_printer ppf path =
-      fprintf ppf "<printer %a raised an exception>" Printtyp.path path
+    let exn_printer ppf path exn =
+      fprintf ppf "<printer %a raised an exception: %s>" Printtyp.path path (Printexc.to_string exn)
 
-    let out_exn path =
-      Oval_printer (fun ppf -> exn_printer ppf path)
+    let out_exn path exn =
+      Oval_printer (fun ppf -> exn_printer ppf path exn)
 
     let install_printer path ty fn =
       let print_val ppf obj =
-        try fn ppf obj with _exn -> exn_printer ppf path in
+        try fn ppf obj with exn -> exn_printer ppf path exn in
       let printer obj = Oval_printer (fun ppf -> print_val ppf obj) in
       printers := (path, Simple (ty, printer)) :: !printers
 
@@ -172,7 +172,7 @@ module Make(O : OBJ)(EVP : EVALPATH with type valu = O.t) = struct
         | Zero fn ->
             let out_printer obj =
               let printer ppf =
-                try fn ppf obj with _ -> exn_printer ppf function_path in
+                try fn ppf obj with exn -> exn_printer ppf function_path exn in
               Oval_printer printer in
             Zero out_printer
         | Succ fn ->
@@ -559,13 +559,13 @@ module Make(O : OBJ)(EVP : EVALPATH with type valu = O.t) = struct
           begin match (Ctype.expand_head env ty).desc with
           | Tconstr (p, args, _) when Path.same p path ->
               begin try apply_generic_printer path (fn depth) args
-              with _ -> (fun _obj -> out_exn path) end
+              with exn -> (fun _obj -> out_exn path exn) end
           | _ -> find remainder end in
       find !printers
 
     and apply_generic_printer path printer args =
       match (printer, args) with
-      | (Zero fn, []) -> (fun (obj : O.t)-> try fn obj with _ -> out_exn path)
+      | (Zero fn, []) -> (fun (obj : O.t)-> try fn obj with exn -> out_exn path exn)
       | (Succ fn, arg :: args) ->
           let printer = fn (fun depth obj -> tree_of_val depth obj arg) in
           apply_generic_printer path printer args
