@@ -143,10 +143,14 @@ and expression_desc =
         (** let P1 = E1 and ... and Pn = EN in E       (flag = Nonrecursive)
             let rec P1 = E1 and ... and Pn = EN in E   (flag = Recursive)
          *)
-  | Texp_function of arg_label * case list * partial
+  | Texp_function of { arg_label : arg_label; param : Ident.t;
+      cases : case list; partial : partial; }
         (** [Pexp_fun] and [Pexp_function] both translate to [Texp_function].
             See {!Parsetree} for more details.
 
+            [param] is the identifier that is to be used to name the
+            parameter of the function.
+
             partial =
               [Partial] if the pattern match is partial
               [Total] otherwise.
